@@ -40,6 +40,8 @@
 #include <net/if.h>
 #include <net/if_arp.h>
 #include <netinet/icmp6.h>
+#include <fcntl.h>
+#include <linux/if_tun.h>
 
 #include "connman.h"
 
@@ -86,13 +88,15 @@ int __connman_inet_modify_address(int cmd, int flags,
 	struct in_addr ipv4_addr, ipv4_dest, ipv4_bcast;
 	int sk, err;
 
-	DBG("");
+	DBG("cmd %#x flags %#x index %d family %d address %s peer %s "
+		"prefixlen %hhu broadcast %s", cmd, flags, index, family,
+		address, peer, prefixlen, broadcast);
 
 	if (address == NULL)
-		return -1;
+		return -EINVAL;
 
 	if (family != AF_INET && family != AF_INET6)
-		return -1;
+		return -EINVAL;
 
 	memset(&request, 0, sizeof(request));
 
@@ -148,7 +152,7 @@ int __connman_inet_modify_address(int cmd, int flags,
 
 	sk = socket(AF_NETLINK, SOCK_DGRAM, NETLINK_ROUTE);
 	if (sk < 0)
-		return -1;
+		return -errno;
 
 	memset(&nl_addr, 0, sizeof(nl_addr));
 	nl_addr.nl_family = AF_NETLINK;
@@ -460,7 +464,7 @@ struct connman_device *connman_inet_create_device(int index)
 	if (devname == NULL)
 		return NULL;
 
-	if (__connman_element_device_isfiltered(devname) == TRUE) {
+	if (__connman_device_isfiltered(devname) == TRUE) {
 		connman_info("Ignoring interface %s (filtered)", devname);
 		free(devname);
 		return NULL;
@@ -539,6 +543,7 @@ struct in6_ifreq {
 int connman_inet_set_ipv6_address(int index,
 		struct connman_ipaddress *ipaddress)
 {
+	int err;
 	unsigned char prefix_len;
 	const char *address;
 
@@ -550,11 +555,12 @@ int connman_inet_set_ipv6_address(int index,
 
 	DBG("index %d address %s prefix_len %d", index, address, prefix_len);
 
-	if ((__connman_inet_modify_address(RTM_NEWADDR,
-			NLM_F_REPLACE | NLM_F_ACK, index, AF_INET6,
-				address, NULL, prefix_len, NULL)) < 0) {
-		connman_error("Set IPv6 address error");
-		return -1;
+	err = __connman_inet_modify_address(RTM_NEWADDR,
+				NLM_F_REPLACE | NLM_F_ACK, index, AF_INET6,
+				address, NULL, prefix_len, NULL);
+	if (err < 0) {
+		connman_error("%s: %s", __func__, strerror(-err));
+		return err;
 	}
 
 	return 0;
@@ -562,6 +568,7 @@ int connman_inet_set_ipv6_address(int index,
 
 int connman_inet_set_address(int index, struct connman_ipaddress *ipaddress)
 {
+	int err;
 	unsigned char prefix_len;
 	const char *address, *broadcast, *peer;
 
@@ -575,11 +582,12 @@ int connman_inet_set_address(int index, struct connman_ipaddress *ipaddress)
 
 	DBG("index %d address %s prefix_len %d", index, address, prefix_len);
 
-	if ((__connman_inet_modify_address(RTM_NEWADDR,
-			NLM_F_REPLACE | NLM_F_ACK, index, AF_INET,
-				address, peer, prefix_len, broadcast)) < 0) {
-		DBG("address setting failed");
-		return -1;
+	err = __connman_inet_modify_address(RTM_NEWADDR,
+				NLM_F_REPLACE | NLM_F_ACK, index, AF_INET,
+				address, peer, prefix_len, broadcast);
+	if (err < 0) {
+		connman_error("%s: %s", __func__, strerror(-err));
+		return err;
 	}
 
 	return 0;
@@ -588,12 +596,15 @@ int connman_inet_set_address(int index, struct connman_ipaddress *ipaddress)
 int connman_inet_clear_ipv6_address(int index, const char *address,
 							int prefix_len)
 {
+	int err;
+
 	DBG("index %d address %s prefix_len %d", index, address, prefix_len);
 
-	if ((__connman_inet_modify_address(RTM_DELADDR, 0, index, AF_INET6,
-					address, NULL, prefix_len, NULL)) < 0) {
-		connman_error("Clear IPv6 address error");
-		return -1;
+	err = __connman_inet_modify_address(RTM_DELADDR, 0, index, AF_INET6,
+				address, NULL, prefix_len, NULL);
+	if (err < 0) {
+		connman_error("%s: %s", __func__, strerror(-err));
+		return err;
 	}
 
 	return 0;
@@ -601,6 +612,7 @@ int connman_inet_clear_ipv6_address(int index, const char *address,
 
 int connman_inet_clear_address(int index, struct connman_ipaddress *ipaddress)
 {
+	int err;
 	unsigned char prefix_len;
 	const char *address, *broadcast, *peer;
 
@@ -611,10 +623,11 @@ int connman_inet_clear_address(int index, struct connman_ipaddress *ipaddress)
 
 	DBG("index %d address %s prefix_len %d", index, address, prefix_len);
 
-	if ((__connman_inet_modify_address(RTM_DELADDR, 0, index, AF_INET,
-				address, peer, prefix_len, broadcast)) < 0) {
-		DBG("address removal failed");
-		return -1;
+	err = __connman_inet_modify_address(RTM_DELADDR, 0, index, AF_INET,
+				address, peer, prefix_len, broadcast);
+	if (err < 0) {
+		connman_error("%s: %s", __func__, strerror(-err));
+		return err;
 	}
 
 	return 0;
@@ -1265,6 +1278,40 @@ done:
 	return err;
 }
 
+int connman_inet_create_tunnel(char **iface)
+{
+	struct ifreq ifr;
+	int i, fd;
+
+	fd = open("/dev/net/tun", O_RDWR);
+	if (fd < 0) {
+		i = -errno;
+		connman_error("Failed to open /dev/net/tun: %s",
+				strerror(errno));
+		return i;
+	}
+
+	memset(&ifr, 0, sizeof(ifr));
+	ifr.ifr_flags = IFF_TUN | IFF_NO_PI;
+
+	for (i = 0; i < 256; i++) {
+		sprintf(ifr.ifr_name, "tun%d", i);
+
+		if (!ioctl(fd, TUNSETIFF, (void *)&ifr))
+			break;
+	}
+
+	if (i == 256) {
+		connman_error("Failed to find available tun device");
+		close(fd);
+		return -ENODEV;
+	}
+
+	*iface = g_strdup(ifr.ifr_name);
+
+	return fd;
+}
+
 struct rs_cb_data {
 	GIOChannel *channel;
 	__connman_inet_rs_cb_t callback;
