@@ -145,6 +145,9 @@ static gboolean check_ipv6_address(const char *address)
 	unsigned char buf[sizeof(struct in6_addr)];
 	int err;
 
+	if (address == NULL)
+		return FALSE;
+
 	err = inet_pton(AF_INET6, address, buf);
 	if (err > 0)
 		return TRUE;
@@ -153,8 +156,9 @@ static gboolean check_ipv6_address(const char *address)
 }
 
 int connman_ipaddress_set_ipv6(struct connman_ipaddress *ipaddress,
-				const char *address, const char *gateway,
-						unsigned char prefix_length)
+				const char *address,
+				unsigned char prefix_length,
+				const char *gateway)
 {
 	if (ipaddress == NULL)
 		return -EINVAL;
@@ -283,6 +287,20 @@ static struct connman_ipaddress *find_ipaddress(struct connman_ipdevice *ipdevic
 	return NULL;
 }
 
+const char *__connman_ipconfig_type2string(enum connman_ipconfig_type type)
+{
+	switch (type) {
+	case CONNMAN_IPCONFIG_TYPE_UNKNOWN:
+		return "unknown";
+	case CONNMAN_IPCONFIG_TYPE_IPV4:
+		return "IPv4";
+	case CONNMAN_IPCONFIG_TYPE_IPV6:
+		return "IPv6";
+	}
+
+	return NULL;
+}
+
 static const char *type2str(unsigned short type)
 {
 	switch (type) {
@@ -1947,7 +1965,7 @@ int __connman_ipconfig_set_config(struct connman_ipconfig *ipconfig,
 		else
 			return connman_ipaddress_set_ipv6(
 					ipconfig->address, address,
-						gateway, prefix_length);
+						prefix_length, gateway);
 		break;
 
 	case CONNMAN_IPCONFIG_METHOD_DHCP:
